"""GraphQLJson type"""

import json
from typing import Any

from graphql.error import GraphQLError
from graphql.language.ast import StringValueNode, ValueNode
from graphql.language.printer import print_ast
from graphql.pyutils import inspect
from graphql.type.definition import GraphQLScalarType


def _is_json_type(value: Any) -> bool:
    return (
        value is None or isinstance(value, (dict, list, int, float, bool))
    )


def _serialize_json(output_value: Any) -> Any:
    if _is_json_type(output_value):
        return output_value

    try:
        if isinstance(output_value, str):
            return json.loads(output_value)
        raise ValueError('JSON must be a string')
    except (TypeError, ValueError, json.JSONDecodeError) as error:
        raise GraphQLError(
            "JSON cannot represent value: " + inspect(output_value)
        ) from error


def _coerce_json(input_value: Any) -> Any:
    if not _is_json_type(input_value):
        raise GraphQLError(
            "JSON cannot represent value: " + inspect(input_value)
        )

    return input_value


def _parse_json_literal(value_node: ValueNode, _variables: Any = None) -> Any:
    if not isinstance(value_node, StringValueNode):
        raise GraphQLError(
            "JSON cannot represent value: " + print_ast(value_node)
        )

    return json.loads(value_node.value)


GraphQLJson = GraphQLScalarType(
    name='JSON',
    description="The `JSON` scalar type",
    serialize=_serialize_json,
    parse_value=_coerce_json,
    parse_literal=_parse_json_literal
)



"""GraphQLLong type"""

from math import isfinite
from typing import Any

from graphql.error import GraphQLError
from graphql.language.ast import IntValueNode, ValueNode
from graphql.language.printer import print_ast
from graphql.pyutils import inspect
from graphql.type.definition import GraphQLScalarType

MAX_LONG = 2 ** 53 - 1
MIN_LONG = -MAX_LONG


def _serialize_long(output_value: Any) -> int:
    if isinstance(output_value, bool):
        return 1 if output_value else 0
    try:
        if isinstance(output_value, int):
            num = output_value
        elif isinstance(output_value, float):
            num = int(output_value)
            if num != output_value:
                raise ValueError
        elif not output_value and isinstance(output_value, str):
            output_value = ""
            raise ValueError
        else:
            num = int(output_value)  # raises ValueError if not an integer
    except (OverflowError, ValueError, TypeError) as error:
        raise GraphQLError(
            "Long cannot represent non-integer value: " + inspect(output_value)
        ) from error
    if not MIN_LONG <= num <= MAX_LONG:
        raise GraphQLError(
            "Long cannot represent non 53-bit signed integer value: "
            + inspect(output_value)
        )
    return num


def _coerce_long(input_value: Any) -> int:
    if not (
        isinstance(input_value, int) and not isinstance(input_value, bool)
    ) and not (
        isinstance(input_value, float)
        and isfinite(input_value)
        and int(input_value) == input_value
    ):
        raise GraphQLError(
            "Long cannot represent non-integer value: " + inspect(input_value)
        )
    if not MIN_LONG <= input_value <= MAX_LONG:
        raise GraphQLError(
            "Long cannot represent non 53-bit signed integer value: "
            + inspect(input_value)
        )
    return int(input_value)


def _parse_long_literal(value_node: ValueNode, _variables: Any = None) -> int:
    """Parse a long integer value node in the AST."""
    if not isinstance(value_node, IntValueNode):
        raise GraphQLError(
            "Long cannot represent non-integer value: " +
            print_ast(value_node),
            value_node,
        )
    num = int(value_node.value)
    if not MIN_LONG <= num <= MAX_LONG:
        raise GraphQLError(
            "Long cannot represent non 32-bit signed integer value: "
            + print_ast(value_node),
            value_node,
        )
    return num


GraphQLLong = GraphQLScalarType(
    name="Long",
    description="The `Long` scalar type represents"
    " non-fractional signed whole numeric values."
    " Int can represent values between -(2^53) and 2^53 - 1.",
    serialize=_serialize_long,
    parse_value=_coerce_long,
    parse_literal=_parse_long_literal,
)



"""jetblack-graphql-types"""

from .datetime_type import GraphQLDateTime
from .duration_type import GraphQLDuration
from .json_type import GraphQLJson
from .long_type import GraphQLLong

__all__ = [
    'GraphQLDateTime',
    'GraphQLDuration',
    'GraphQLJson',
    'GraphQLLong'
]



"""GraphQLDateTime type"""

from datetime import datetime
from typing import Any, Optional

from graphql.error import GraphQLError
from graphql.language import StringValueNode, ValueNode, print_ast
from graphql.type.definition import GraphQLScalarType
from graphql.pyutils import inspect

from jetblack_iso8601 import iso8601_to_datetime


def _serialize_datetime(output_value: Any) -> datetime:
    if isinstance(output_value, datetime):
        return output_value

    try:
        if isinstance(output_value, str):
            value = iso8601_to_datetime(output_value)
            if value is not None:
                return value
        raise ValueError("Invalid datetime")
    except ValueError as error:
        raise GraphQLError(
            "DateTime cannot represent value: " + inspect(output_value)
        ) from error


def _coerce_datetime(input_value: Any) -> datetime:
    if isinstance(input_value, datetime):
        return input_value

    try:
        if isinstance(input_value, str):
            value = iso8601_to_datetime(input_value)
            if value is not None:
                return value
        raise ValueError('Unable to parse datetime')
    except ValueError as error:
        raise GraphQLError(
            "DateTime cannot represent value: " + inspect(input_value)
        ) from error


def _parse_datetime_literal(
        value_node: ValueNode,
        _variables: Optional[Any] = None
) -> datetime:
    if not isinstance(value_node, StringValueNode):
        raise GraphQLError(
            "DateTime cannot represent non-string value: " +
            print_ast(value_node)
        )
    value = iso8601_to_datetime(value_node.value)
    if value is not None:
        return value

    raise GraphQLError(
        "DateTime cannot represent  value: " +
        print_ast(value_node)
    )


GraphQLDateTime = GraphQLScalarType(
    name="DateTime",
    description="The `DateTime` type represents ISO 8601 datetimes",
    serialize=_serialize_datetime,
    parse_value=_coerce_datetime,
    parse_literal=_parse_datetime_literal
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jetblack_graphql_types']

package_data = \
{'': ['*']}

install_requires = \
['graphql-core>=3.1.6,<4.0.0', 'jetblack-iso8601>=1.0,<2.0']

setup_kwargs = {
    'name': 'jetblack-graphql-types',
    'version': '0.2.0',
    'description': 'Serialization for JSON and XML using typing',
    'long_description': '# jetblack-graphql-types\n\nTypes for graphql\n',
    'author': 'Rob Blackbourn',
    'author_email': 'rob.blackbourn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rob-blackbourn/jetblack-graphql-types',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
